﻿

module("VOICE")

ASSERT(type(voice_debug_log) == "function", "Missing voice_debug_log function.")
ASSERT(type(voice_debug_log_table) == "function", "Missing voice_debug_log_table function.")

ASSERT(type(wjoin) == "function", "Missing wjoin function.")
ASSERT(type(wsplit) == "function", "Missing wsplit function.")
ASSERT(type(getFirstNChar) == "function", "Missing getFirstNChar function.")
ASSERT(type(getLastNChar) == "function", "Missing getLastNChar function.")

ASSERT(type(transform_and_format) == "function", "Missing transform_and_format function.")
ASSERT(type(transform_chain) == "function", "Missing transform_chain function.")
ASSERT(type(table_concat) == "function", "Missing table_concat function.")
ASSERT(type(transform_pattern_match) == "function", "Missing transform_pattern_match function.")


local prepositions = {
	L"auf das ",
	L"auf den ",
	L"auf die ",
	L"in das ",
	L"in den ",
	L"in die ",
	L"durch den ",
	L"über die ",
	L"und dem ",
	L"und der ",
	L"richtung ",
	L"auf ",
}

local preposition_accusatives = {
	{L"auf das ", {L"dreieck$", L"dreieck +%a+", L"kreuz$", L"kreuz +%a+"}},
	{L"auf den ", {L"platz$", L"ring$", L"damm$", L"berg$", L"kanal$", L"^platz de[rs] "}},
	{L"auf die ", {L"allee$", L"treppe$", L"steige$", L"anschlussstelle", L"^%a?%d+%a?$"}},
	{L"in das ", {L"tor$"}},
	{L"in den ", {L"horst$", L"kamp$", L"winkel$", L"pfad$", L"weg$", L"steg$", L"stieg$", L"hof$"}},
	{L"in die ", {L"gasse$", L"strase$", L"strasse$", L"sträse$", L"straße$", L"sträße$", L"^stra[ßs]+e de[rs] "}},
	{L"durch den ", {L"tunnel$", L"tunel$"}},
	{L"über die ", {L"brucke$", L"brücke$"}},
	{L"auf ", {L""}},
}

local preposition_datives = {
	{L"und dem ", {L"dreieck$", L"dreieck +%a+", L"kreuz$", L"kreuz +%a+"}},
	{L"und dem ", {L"platz$", L"ring$", L"damm$", L"berg$", L"kanal$", L"^platz de[rs] "}},
	{L"und der ", {L"allee$", L"treppe$", L"steige$", L"anschlussstelle", L"^%a?%d+%a?$"}},
	{L"und dem ", {L"tor$"}},
	{L"und dem ", {L"horst$", L"kamp$", L"winkel$", L"pfad$", L"weg$", L"steg$", L"stieg$", L"hof$"}},
	{L"und der ", {L"gasse$", L"strase$", L"strasse$", L"sträse$", L"straße$", L"sträße$", L"^stra[ßs]+e de[rs] "}},
	{L"und dem ", {L"tunnel$", L"tunel$"}},
	{L"und der ", {L"brucke$", L"brücke$"}},
	{L"auf ", {L"und "}},
	{L"auf ", {L""}},
}

local prepositions2traffic = {
	L"auf dem ",
	L"auf der ",
	L"im ",
	L"in der ",
	L"am ",
	L"an der ",
	L"richtung ",
	L"auf ",
}

local traffic_preposition_insert_tbl = {
    {L"auf dem ", {L"platz$", L"ring$", L"damm$", L"berg$", L"kanal$", L"^platz de[rs] ", L"dreieck$", L"kreuz$"}},
    {L"auf der ", {L"allee$", L"treppe$", L"steige$", L"^%a?%d+%a?$", L"brucke$", L"brücke$"}},
    {L"im ", {L"horst$", L"kamp$", L"winkel$", L"pfad$", L"weg$", L"steg$", L"stieg$", L"hof$", L"tunnel$", L"tunel$", L"tor$"}},
    {L"in der ", {L"gasse$", L"strase$", L"strasse$", L"sträse$", L"straße$", L"sträße$", L"^stra[ßs]+e de[rs] "}},
    {L"auf ", {L""}},
} 

local traffic_preposition_from_tbl = {
    {L"am ", {L"platz$", L"ring$", L"damm$", L"berg$", L"kanal$", L"^platz de[rs] ", L"dreieck$", L"kreuz$"}},
    {L"an der ", {L"allee$", L"treppe$", L"steige$", L"^%a?%d+%a?$", L"brucke$", L"brücke$"}},
    {L"am ", {L"horst$", L"kamp$", L"winkel$", L"pfad$", L"weg$", L"steg$", L"stieg$", L"hof$", L"tunnel$", L"tunel$", L"tor$"}},
    {L"an der ", {L"gasse$", L"strase$", L"strasse$", L"sträse$", L"straße$", L"sträße$", L"^stra[ßs]+e de[rs] "}},
    {L"bei ", {L""}},
} 
	
local replace_mapinfo = {
	{L"[sS][tT]%.", L"Sankt "},
	{L"^(%u)(%u)[ -]*(%d+)", L"%1.%2. %3"},
	{L"-", L" "},
	{L"(%d)(%a)", L"%1 %2"},
	{L"(%a)(%d)", L"%1 %2"},
	{L"//", L", "},
}
	
local replace_exitnumber = {
	{L"(%d+)(%a)$", L"%1 %2"},
	{L"(%d+)$", L"%1"},
	{L"(%a)(%d+)$", L"%1 %2"},
}

local replace_sentence={
	{L"%. *[Dd]ann", L", dann"},
	{L"%. *[Dd]anach", L", danach"},
	{L"%. *Sie kommen dann", L", Sie kommen dann"},
	{L"(%p)+ *(%p)+", L"%1 "},
	{L"%s+(%p+)", L"%1 "},
	{L" +", L" "},
	{L"(%d+)(%.)(%d+)", L"%1 punkt %3"},
	{L"(%d+)(%.)", L"%1 %2"},
}

local exitPreposition = L"an der Ausfahrt "
local towardsPreposition = L"richtung"
local sameRoadPostposition = L" weiter folgen "

local function find_preposition(str, tbl)
	str = wstring.lower(str)
	for k, value in ipairs(tbl) do
		for _, pattern in ipairs(tbl[k][2]) do
			if wstring.find(str, pattern) then
				return tbl[k][1] 
			end	
		end		
	end
	return L""
end

local function get_prePosition(data, basePrePositions, prepositionTable)
	local str = wstring.lower(data.text)
	local _, found_preposition = transform_pattern_match(str, basePrePositions)
	
	if data.phoneme then
		if found_preposition then
			return L""
		else
			return find_preposition(data.text, prepositionTable)
		end
	else
		if found_preposition then
			str = wstring.gsub(str, found_preposition, L"")
			data.text = str
			return find_preposition(str, prepositionTable)
		else
			return find_preposition(data.text, prepositionTable)
		end
	end
end

local function getTowardsPreposition(data)
	if wstring.find(data.text, towardsPreposition) then
		return L""
	else
		return towardsPreposition .. L" "
	end
end

local function get_roadNameToThen(data, idx)
	if data[idx+1] and data[idx+1].road and data[idx+1].road.name then
		return data[idx+1].road.name
	end
end

local function roadNumberName(data, idx)
	if data[idx].road then
		local roadNumber = data[idx].road.number
		local roadName = data[idx].road.name
		local roadNameToThen = get_roadNameToThen(data, idx)
		
		if roadNumber and roadName then
			if (roadNumber.change or roadNumber.change == nil) then
				return get_prePosition(roadNumber, prepositions, preposition_accusatives) .. transform_format_roadnumber_eu(roadNumber.text) .. L", " .. transform_and_format(roadName, replace_mapinfo)
			else
				local outStr = get_prePosition(roadNumber, prepositions, preposition_datives) .. transform_format_roadnumber_eu(roadNumber.text)
				if roadNameToThen and (roadName.text == roadNameToThen.text) then
					return outStr .. L", " .. transform_and_format(roadName, replace_mapinfo) .. sameRoadPostposition
				else
					return outStr .. sameRoadPostposition
				end
			end
		elseif roadNumber then
			if (roadNumber.change or roadNumber.change == nil) then
				return get_prePosition(roadNumber, prepositions, preposition_accusatives) .. transform_format_roadnumber_eu(roadNumber.text)
			else
				return get_prePosition(roadNumber, prepositions, preposition_datives) .. transform_format_roadnumber_eu(roadNumber.text) .. sameRoadPostposition
			end
		elseif roadName then
			if (roadName.change or roadName.change == nil) then
				return get_prePosition(roadName, prepositions, preposition_accusatives) .. transform_and_format(roadName, replace_mapinfo)
			else
				return get_prePosition(roadName, prepositions, preposition_datives) .. transform_and_format(roadName, replace_mapinfo) .. sameRoadPostposition
			end
		end
	end
end

local function signpost_exitNumberName(data, idx)
	if data[idx].signpost then
		local exitNumber = data[idx].signpost.exitnumber
		local exitName = data[idx].signpost.exitname
		if exitNumber and exitName then
			return exitPreposition .. transform_and_format(exitNumber, replace_exitnumber) .. L", " .. transform_and_format(exitName, replace_mapinfo)
		elseif exitNumber then
			return exitPreposition .. transform_and_format(exitNumber, replace_exitnumber)
		elseif exitName then
			return exitPreposition .. transform_and_format(exitName, replace_mapinfo)
		end
	end
end

local function signpost_roadnumber(data, idx)
	if data[idx].signpost then
		local roadNumber = data[idx].signpost.roadnumber
		
		local signRoadBranch = data[idx].signpost.sign_road_branch
		
		if roadNumber and signRoadBranch and (roadNumber.change or roadNumber.change == nil) then
			return find_preposition(roadNumber.text, preposition_accusatives) .. transform_format_roadnumber_eu(roadNumber.text)
		
		elseif roadNumber and signRoadBranch and not roadNumber.change then
			return find_preposition(roadNumber.text, preposition_datives) .. transform_format_roadnumber_eu(roadNumber.text) .. sameRoadPostposition
		
		elseif roadNumber and not signRoadBranch then
			if data[idx].road then
				if (data[idx].road.number and data[idx].road.number.text ~= roadNumber.text) or data[idx].road.name then
					return roadNumberName(data, idx) .. L", " .. getTowardsPreposition(roadNumber) .. transform_format_roadnumber_eu(roadNumber.text)
				else
					return roadNumberName(data, idx)
				end
			else
				return getTowardsPreposition(roadNumber) .. transform_format_roadnumber_eu(roadNumber.text)
			end
		else
			if data[idx].road then
				return roadNumberName(data, idx)
			end
		end
	end
end

local function signpost_destination(data, idx)
	if data[idx].signpost then
		local dest = data[idx].signpost.destination
		if dest then
			return getTowardsPreposition(dest) .. transform_and_format(dest, replace_mapinfo)
		end
	end
end

local function signpost_settlement(data, idx)
	if data[idx].signpost then
		local settlement = data[idx].signpost.settlement
		if settlement then
			return getTowardsPreposition(settlement) .. transform_and_format(settlement, replace_mapinfo)
		end
	end
end

function format_destname(data, idx)
	local t = {}
	if data[idx].signpost then
		t = transform_chain(t,  signpost_exitNumberName,	data, idx)
		t = transform_chain(t,  signpost_roadnumber, 		data, idx)
		t = transform_chain(t,  signpost_destination,		data, idx)
		t = transform_chain(t,  signpost_settlement, 		data, idx)
	elseif data[idx].road then
		t = transform_chain(t,  roadNumberName, 			data, idx)
	end
	return table_concat(t, L", ")
end

function format_streetname(data, idx)
	local t = {}
	if data[idx].road then
		t = transform_chain(t,  roadNumberName, 				data, idx)
	end
	return table_concat(t, L", ")
end

function format_sentence(str)
	for key, value in ipairs(replace_sentence) do
		str = wstring.gsub(str, value[1], value[2])
	end
	return str
end


function route_summary_format_road_name(data)
	if data then
		
		return transform_and_format(data, transform_format_roadnumber_eu, nil, true)
	end
end

function route_summary_format_street_name(data)
	if data then
		return transform_and_format(data)
	end
end

function route_summary_format_bridge_tunnel(data)
	if data then
		return transform_and_format(data)
	end
end

function route_summary_format_order(data)
	if data then
		return transform_and_format(data)
	end
end



function traffic_event_supported()
	return true
end

function traffic_event(DescKey, data)
	local str = translate_voice(DescKey)
	local loc = MODEL.regional.is_it_voice_localizable(DescKey) 
	ASSERT(loc, "Missing TrafficEvent dictionary.voice key:" .. DescKey)
	voice_debug_log(L"Desc: " .. str)
	
	local road=L""
	
	if data.roadnumber and data.roadnumber.text and data.country and data.country.text==L"_GER" then
		ASSERT(data.roadnumber.text:len())
		if data.roadnumber.text:len() and data.roadnumber.text:sub(1, 1)~=L"A" and  data.roadnumber.text:sub(1, 1)~=L"B" then
			data.roadnumber=nil
		end
	end
	
	

	if data.roadnumber then
		road = L" auf der " .. transform_format_roadnumber_eu(data.roadnumber.text)
	
	elseif data.roadname then
		road = transform_and_format(data.roadname)
		road = find_preposition(data.roadname.text, traffic_preposition_insert_tbl) .. road
	end
	str = str .. road
	
	if data.from and not data.to then
		local from =data.from.text
		local to=L""
		if not data.to then
		
		local _, _, f, t = wstring.find(data.from.text, L"^([^,]+),(.+)$")
			if f then 
				from=f
				to=t
			end
		end
		
		if from and to == L"" then
			str =  str .. L" , bei " .. transform_and_format(from)
		else
			str =  str .. L" , zwischen " .. transform_and_format(from) .. L" und " .. transform_and_format(to)
		end	
	elseif data.from and data.to then 
		
			str =  str .. L" , zwischen " .. transform_and_format(data.from) .. L" und " .. transform_and_format(data.to)
	
	end
	str = str .. L"."
	voice_debug_log(L"Sentence to say: " .. str)
	return str
end

